@@ -1,6 +1,7 @@
-/* Copyright (C) 2015-2022 Ben Collins <bcollins@maclara-llc.com>
+/* Copyright (C) 2015-2023 Ben Collins <bcollins@maclara-llc.com>
    This file is part of the JWT C Library
 
+   SPDX-License-Identifier:  MPL-2.0
    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -571,6 +572,20 @@ JWT_EXPORT int jwt_dump_fp(jwt_t *jwt, FILE *fp, int pretty);
  */
 JWT_EXPORT char *jwt_dump_str(jwt_t *jwt, int pretty);
 
+/**
+ * Return plain text representation of grants as a string.
+ *
+ * Similar to jwt_dump_str() except that only a string containing the
+ * grants string is returned. The string must be freed by the caller.
+ *
+ * @param jwt Pointer to a JWT object.
+ * @param pretty Enables better visual formatting of output. Generally only
+ *     used for debugging.
+ * @return A nul terminated string on success, NULL on error with errno
+ *     set appropriately.
+ */
+JWT_EXPORT char *jwt_dump_grants_str(jwt_t *jwt, int pretty);
+
 /**
  * Fully encode a JWT object and write it to FILE.
  *
@@ -714,7 +729,7 @@ JWT_EXPORT void jwt_get_alloc(jwt_malloc_t *pmalloc, jwt_realloc_t *prealloc, jw
  /** @} */
 
 /**
- * @defgroup jwt_vaildate JWT validation functions
+ * @defgroup jwt_validate JWT validation functions
  * These functions allow you to define requirements for JWT validation.
  *
  * The most basic validation is that the JWT uses the expected algorithm.
@@ -981,6 +996,21 @@ JWT_EXPORT int jwt_valid_set_exp_leeway(jwt_valid_t *jwt_valid, const time_t exp
  */
 JWT_EXPORT int jwt_valid_set_headers(jwt_valid_t *jwt_valid, int hdr);
 
+/**
+ * Parses exceptions and returns a comma delimited and human-readable string.
+ *
+ * The returned string must be freed by the caller. If you changed the allocation
+ * method using jwt_set_alloc, then you must use jwt_free_str() to free the memory.
+ *
+ * Note: This string is currently en-US ASCII only. Language support will come in the
+ * future.
+ *
+ * @param exceptions Integer containing the exception flags.
+ * @return A null terminated string on success, NULL on error with errno
+ *     set appropriately.
+ */
+JWT_EXPORT char *jwt_exception_str(unsigned int exceptions);
+
 /** @} */
 
 #ifdef __cplusplus
